'' =================================================================================================
''
''   File....... jm_rn-42_control.spin
''   Purpose.... Simple control demo using RN-42 Bluetooth module 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 26 JAN 2015 
''
'' =================================================================================================

{

  Connections:

    Set RN-42 power jumper to 3.3V

    P15 <-- RN-42.TX
    P14 --> RN-42.TX

    Connect RN-42.RTS to RN-42.CTS

    Connect RN-42 to 3.3V and GND rails

}


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = _xinfreq * (_clkmode >> 6)                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us
                                                                  
                                                                  
con { activity board io pins }                                    
                                                                  
  RX1     = 31                                                   ' programming / terminal
  TX1     = 30                                                    
                                                                  
  SDA     = 29                                                   ' eeprom / i2c
  SCL     = 28                                                    
                                                                  
  AUD_RT  = 27                                                   ' audio (d/a) outputs
  AUD_LF  = 26                                                    
                                                                  
  LED_1   = 27                                                   ' leds (on audio outputs)
  LED_L   = 26                                                    
                                                                  
  SD_CS   = 25                                                   ' usd card
  SD_DI   = 24                                                    
  SD_CLK  = 23                                                    
  SD_DO   = 22                                                    
                                                                  
  ADC_CS  = 21                                                   ' adc input
  ADC_CLK = 20                                                    
  ADC_DO  = 19 { miso }                                           
  ADC_DI  = 18 { mosi }                                           
                                                                  
  SVO_6   = 17                                                   ' servo headers
  SVO_5   = 16                                                    
                                                                  
  SVO_4   = 15                                                    
  SVO_3   = 14                                                    
                                                                  
  SVO_2   = 13                                                    
  SVO_1   = 12                                                    

  
  RX2     = 15                                                   ' to rn-42 tx pin (3.3v mode)
  TX2     = 14                                                   ' to rn-42 rx pin


con 

  #(-1), IS_ON, IS_OFF

  BUF_SIZE = 80
  TOK_SIZE = 12

  N_TOKENS =  5


obj

  term  : "jm_fullduplexserial"                                  ' * serial for terminal
  btcom : "jm_fullduplexserial"                                  ' * serial for rn-42
  adc   : "jm_adc124s021"                                        '   for onboard adc
  io    : "jm_io"                                                '   basic io
  time  : "jm_time"                                              '   delays/timing
  str   : "jm_strings"
                                                                  
' * uses cog when loaded                                          
                                                                  
        
var

  byte  buffer[80]                                               ' raw input buffer

  byte  token0[TOK_SIZE]                                         ' token buffers
  byte  token1[TOK_SIZE]
  byte  token2[TOK_SIZE]  
  byte  token3[TOK_SIZE]
  byte  token4[TOK_SIZE]
  byte  token5[TOK_SIZE] 
  byte  token6[TOK_SIZE] 
  byte  token7[TOK_SIZE] 
  byte  token8[TOK_SIZE] 
  byte  token9[TOK_SIZE] 


pub main | idx, c 

  setup

  term.tx(term#CLS)
  term.str(string("Bluetooth Command Processor", 13, 13))

  repeat
    bytefill(@buffer, 0, BUF_SIZE)                               ' clear buffer
    idx := 0                                                     ' reset buffer index
    
    repeat
      buffer[idx++] := (c := btcom.rx)                           ' rx byte(s) from bluetooth
      if (c == 13)                                               ' if CR we're done 
        quit                                                 

    process_buf(@buffer)
        

pub process_buf(p_buf) | count, n, p_tok

'' Process the command buffer
'' -- presently parses and reports tokens

  str.ucstr(p_buf)                                               ' conert to uppercase  

  term.str(string("BT> "))
  term.str(p_buf)
  term.str(string("    -- length: "))
  term.dec(strsize(p_buf))
  term.tx(13)
  term.str(string("    -- tokens: "))
  count := count_tokens(p_buf)
  term.dec(count)
  term.tx(13)

  repeat n from 0 to count-1
    p_tok := @token0 + (n * TOK_SIZE)                            ' point to token buffer
    extract_token(@buffer, p_tok, n)                             ' extract token
    term.str(string("    -- token"))                             ' report
    term.dec(n)
    term.str(string(": <"))
    term.str(p_tok)
    term.tx(">")
    term.tx(13)

  term.tx(13)


  ' handle a couple commands

  if (strcomp(@token0, string("P26")))
    if (strcomp(@token1, string("0")))
      io.low(26)
    elseif (strcomp(@token1, string("OFF")))    
      io.low(26) 
    elseif (strcomp(@token1, string("LOW")))    
      io.low(26) 
    elseif (strcomp(@token1, string("1")))    
      io.high(26) 
    elseif (strcomp(@token1, string("ON")))    
      io.high(26)
    elseif (strcomp(@token1, string("HIGH")))    
      io.high(26)
    else
      ' bad token1 - consider NAK to master

      
  if (strcomp(@token0, string("P27")))
    if (strcomp(@token1, string("0")))
      io.low(27)
    elseif (strcomp(@token1, string("OFF")))    
      io.low(27) 
    elseif (strcomp(@token1, string("LOW")))    
      io.low(27) 
    elseif (strcomp(@token1, string("1")))    
      io.high(27) 
    elseif (strcomp(@token1, string("ON")))    
      io.high(27)
    elseif (strcomp(@token1, string("HIGH")))    
      io.high(27)
    else
      ' bad token1 - consider NAK to master
      

pub count_tokens(p_str) | c, tokens, tflag

  tokens := 0                                                    ' clear count
  tflag := false                                                 ' not inside token

  repeat strsize(p_str)                                          ' iterate through string
    c := byte[p_str++]
    if (str.is_alphanum(c))                                      ' if letter or number
      if (tflag == false)                                        '  and not already in token
        ++tokens                                                 '  update token count
        tflag := true                                            '  mark in token
    else
      tflag := false                                             ' mark not in token
    
  return tokens


pub extract_token(p_str, p_tok, n) | c, tokens, tflag

'' Extract token from buffer at p_str
'' -- p_tok is pointer to token buffer
'' -- n is the target token (0..9)

  tflag := false                                                 ' not inside token

  repeat strsize(p_str)                                          ' iterate through buffer
    c := byte[p_str++]                                           ' get a character
    ifnot (str.is_alphanum(c))                                   ' if not letter or number
      tflag := false                                             ' mark not in token
    else 
      if (tflag == false)                                        ' if not in character and letter or number
        tflag := true                                            ' mark in token
        if (n > 0)                                               ' if not at target
          --n                                                    ' decrement 
        else                                                     ' token found - extract it
          bytefill(p_tok, 0, TOK_SIZE)                           ' clear token buffer
          byte[p_tok++] := c                                     ' put current character into token buffer
          repeat
            c := byte[p_str++]                                   ' get next character
            if (str.is_alphanum(c))                              ' if letter or number
              byte[p_tok++] := c                                 ' add to token buffer
            else
              return                                             ' we're done        


pub setup

'' Setup objects for project

  term.start(RX1, TX1, %0000, 115_200)                           ' start serial for terminal
                                                                  
  btcom.start(RX2, TX2, %0000, 9_600)                            ' start serial for rn-42 (bluetooth)
                                                                  
  adc.start(ADC_CS, ADC_CLK, ADC_DI, ADC_DO)                     ' connect to adc
                                                                  
  time.start                                                     ' for pause and timing
  time.pause(2000)                                               ' let bt module start
                                                                  

dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}